@@ -1,9 +1,10 @@
+#!/bin/sh
 # Convenience script for regenerating all aclocal.m4, config.h.in, Makefile.in,
 # configure files with new versions of autoconf or automake.
 #
-# This script requires autoconf-2.58..2.59 and automake-1.8.2..1.9 in the PATH.
+# This script requires autoconf-2.60 and automake-1.8.2..1.9 in the PATH.
 
-# Copyright (C) 2003-2004 Free Software Foundation, Inc.
+# Copyright (C) 2003-2006 Free Software Foundation, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -52,4 +53,11 @@ cp -p gettext-runtime/ABOUT-NLS gettext-tools/ABOUT-NLS
  automake
 )
 
+(cd gettext-tools/examples
+ aclocal -I ../../gettext-runtime/m4 -I ../../m4
+ autoconf
+ automake
+ ./configure && (cd po && make update-po) && make distclean
+)
+
 cp -p autoconf-lib-link/config.rpath build-aux/config.rpath
