@@ -1,5 +1,5 @@
 /* Multiline error-reporting functions.
-   Copyright (C) 2001-2003 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2006 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -28,10 +28,6 @@ extern "C" {
 #endif
 
 
-/* Format a message and return the freshly allocated resulting string.  */
-extern char *xasprintf (const char *format, ...)
-     __attribute__ ((__format__ (__printf__, 1, 2)));
-
 /* Emit a multiline warning to stderr, consisting of MESSAGE, with the
    first line prefixed with PREFIX and the remaining lines prefixed with
    the same amount of spaces.  Reuse the spaces of the previous call if
