@@ -1,5 +1,5 @@
 /* Message list concatenation and duplicate handling.
-   Copyright (C) 2001-2003 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2005-2006 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -31,6 +31,7 @@
 
 #include "error.h"
 #include "xerror.h"
+#include "xvasprintf.h"
 #include "message.h"
 #include "read-po.h"
 #include "po-charset.h"
@@ -69,7 +70,7 @@ is_message_selected (const message_ty *tmp)
 {
   int used = (tmp->used >= 0 ? tmp->used : - tmp->used);
 
-  return (tmp->msgid[0] == '\0'
+  return (is_header (tmp)
 	  ? !omit_header	/* keep the header entry */
 	  : (used > more_than && used < less_than));
 }
@@ -79,7 +80,7 @@ static bool
 is_message_needed (const message_ty *mp)
 {
   if (!msgcomm_mode
-      && ((mp->msgid[0] != '\0' && mp->is_fuzzy) || mp->msgstr[0] == '\0'))
+      && ((!is_header (mp) && mp->is_fuzzy) || mp->msgstr[0] == '\0'))
     /* Weak translation.  Needed if there are only weak translations.  */
     return mp->tmp->used < 0 && is_message_selected (mp->tmp);
   else
@@ -137,7 +138,7 @@ catenate_msgdomain_list (string_list_ty *file_list, const char *to_code)
 	  if (mlp->nitems > 0)
 	    {
 	      for (j = 0; j < mlp->nitems; j++)
-		if (mlp->item[j]->msgid[0] == '\0' && !mlp->item[j]->obsolete)
+		if (is_header (mlp->item[j]) && !mlp->item[j]->obsolete)
 		  {
 		    const char *header = mlp->item[j]->msgstr;
 
@@ -230,7 +231,7 @@ domain \"%s\" in input file `%s' doesn't contain a header entry with a charset s
 	  char *project_id = NULL;
 
 	  for (j = 0; j < mlp->nitems; j++)
-	    if (mlp->item[j]->msgid[0] == '\0' && !mlp->item[j]->obsolete)
+	    if (is_header (mlp->item[j]) && !mlp->item[j]->obsolete)
 	      {
 		const char *header = mlp->item[j]->msgstr;
 
@@ -296,11 +297,11 @@ domain \"%s\" in input file `%s' doesn't contain a header entry with a charset s
 	      message_ty *tmp;
 	      size_t i;
 
-	      tmp = message_list_search (total_mlp, mp->msgid);
+	      tmp = message_list_search (total_mlp, mp->msgctxt, mp->msgid);
 	      if (tmp == NULL)
 		{
-		  tmp = message_alloc (mp->msgid, mp->msgid_plural, NULL, 0,
-				       &mp->pos);
+		  tmp = message_alloc (mp->msgctxt, mp->msgid, mp->msgid_plural,
+				       NULL, 0, &mp->pos);
 		  tmp->is_fuzzy = true; /* may be set to false later */
 		  for (i = 0; i < NFORMATS; i++)
 		    tmp->is_format[i] = undecided; /* may be set to yes/no later */
@@ -312,7 +313,7 @@ domain \"%s\" in input file `%s' doesn't contain a header entry with a charset s
 		}
 
 	      if (!msgcomm_mode
-		  && ((mp->msgid[0] != '\0' && mp->is_fuzzy)
+		  && ((!is_header (mp) && mp->is_fuzzy)
 		      || mp->msgstr[0] == '\0'))
 		/* Weak translation.  Counted as negative tmp->used.  */
 		{
@@ -470,8 +471,20 @@ To select a different output encoding, use the --to-code option.\n\
 	       conversion that would only replace the charset name in the
 	       header entry with its canonical equivalent.  */
 	    if (!(to_code == NULL && canon_charsets[n][k] == canon_to_code))
-	      iconv_message_list (mdlp->item[k]->messages, canon_charsets[n][k],
-				  canon_to_code, files[n]);
+	      if (iconv_message_list (mdlp->item[k]->messages,
+				      canon_charsets[n][k], canon_to_code,
+				      files[n]))
+		{
+		  multiline_error (xstrdup (""),
+				   xasprintf (_("\
+Conversion of file %s from %s encoding to %s encoding\n\
+changes some msgids or msgctxts.\n\
+Either change all msgids and msgctxts to be pure ASCII, or ensure they are\n\
+UTF-8 encoded from the beginning, i.e. already in your source code files.\n"),
+					      files[n], canon_charsets[n][k],
+					      canon_to_code));
+		  exit (EXIT_FAILURE);
+		}
       }
 
   /* Fill the resulting messages.  */
