@@ -1,5 +1,5 @@
 /* Message list charset and locale charset handling.
-   Copyright (C) 2001-2003 Free Software Foundation, Inc.
+   Copyright (C) 2001-2003, 2005-2006 Free Software Foundation, Inc.
    Written by Bruno Haible <haible@clisp.cons.org>, 2001.
 
    This program is free software; you can redistribute it and/or modify
@@ -25,7 +25,6 @@
 /* Specification.  */
 #include "msgl-iconv.h"
 
-#include <errno.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
@@ -34,16 +33,17 @@
 # include <iconv.h>
 #endif
 
-#include "error.h"
 #include "progname.h"
 #include "basename.h"
 #include "message.h"
 #include "po-charset.h"
+#include "iconvstring.h"
 #include "msgl-ascii.h"
 #include "xalloc.h"
 #include "xallocsa.h"
 #include "strstr.h"
-#include "exit.h"
+#include "xvasprintf.h"
+#include "po-xerror.h"
 #include "gettext.h"
 
 #define _(str) gettext (str)
@@ -51,128 +51,32 @@
 
 #if HAVE_ICONV
 
-/* Converts an entire string from one encoding to another, using iconv.
-   Return value: 0 if successful, otherwise -1 and errno set.  */
-static int
-iconv_string (iconv_t cd, const char *start, const char *end,
-	      char **resultp, size_t *lengthp)
+static void conversion_error (const struct conversion_context* context)
+#if defined __GNUC__ && ((__GNUC__ == 2 && __GNUC_MINOR__ >= 5) || __GNUC__ > 2)
+     __attribute__ ((noreturn))
+#endif
+;
+static void
+conversion_error (const struct conversion_context* context)
 {
-#define tmpbufsize 4096
-  size_t length;
-  char *result;
-
-  /* Avoid glibc-2.1 bug and Solaris 2.7-2.9 bug.  */
-# if defined _LIBICONV_VERSION \
-    || !((__GLIBC__ - 0 == 2 && __GLIBC_MINOR__ - 0 <= 1) || defined __sun)
-  /* Set to the initial state.  */
-  iconv (cd, NULL, NULL, NULL, NULL);
-# endif
-
-  /* Determine the length we need.  */
-  {
-    size_t count = 0;
-    char tmpbuf[tmpbufsize];
-    const char *inptr = start;
-    size_t insize = end - start;
-
-    while (insize > 0)
-      {
-	char *outptr = tmpbuf;
-	size_t outsize = tmpbufsize;
-	size_t res = iconv (cd,
-			    (ICONV_CONST char **) &inptr, &insize,
-			    &outptr, &outsize);
-
-	if (res == (size_t)(-1))
-	  {
-	    if (errno == E2BIG)
-	      ;
-	    else if (errno == EINVAL)
-	      break;
-	    else
-	      return -1;
-	  }
-# if !defined _LIBICONV_VERSION && (defined sgi || defined __sgi)
-	/* Irix iconv() inserts a NUL byte if it cannot convert.  */
-	else if (res > 0)
-	  return -1;
-# endif
-	count += outptr - tmpbuf;
-      }
-    /* Avoid glibc-2.1 bug and Solaris 2.7 bug.  */
-# if defined _LIBICONV_VERSION \
-    || !((__GLIBC__ - 0 == 2 && __GLIBC_MINOR__ - 0 <= 1) || defined __sun)
-    {
-      char *outptr = tmpbuf;
-      size_t outsize = tmpbufsize;
-      size_t res = iconv (cd, NULL, NULL, &outptr, &outsize);
-
-      if (res == (size_t)(-1))
-	return -1;
-      count += outptr - tmpbuf;
-    }
-# endif
-    length = count;
-  }
-
-  *lengthp = length;
-  *resultp = result = xrealloc (*resultp, length);
-  if (length == 0)
-    return 0;
-
-  /* Avoid glibc-2.1 bug and Solaris 2.7-2.9 bug.  */
-# if defined _LIBICONV_VERSION \
-    || !((__GLIBC__ - 0 == 2 && __GLIBC_MINOR__ - 0 <= 1) || defined __sun)
-  /* Return to the initial state.  */
-  iconv (cd, NULL, NULL, NULL, NULL);
-# endif
-
-  /* Do the conversion for real.  */
-  {
-    const char *inptr = start;
-    size_t insize = end - start;
-    char *outptr = result;
-    size_t outsize = length;
-
-    while (insize > 0)
-      {
-	size_t res = iconv (cd,
-			    (ICONV_CONST char **) &inptr, &insize,
-			    &outptr, &outsize);
-
-	if (res == (size_t)(-1))
-	  {
-	    if (errno == EINVAL)
-	      break;
-	    else
-	      return -1;
-	  }
-# if !defined _LIBICONV_VERSION && (defined sgi || defined __sgi)
-	/* Irix iconv() inserts a NUL byte if it cannot convert.  */
-	else if (res > 0)
-	  return -1;
-# endif
-      }
-    /* Avoid glibc-2.1 bug and Solaris 2.7 bug.  */
-# if defined _LIBICONV_VERSION \
-    || !((__GLIBC__ - 0 == 2 && __GLIBC_MINOR__ - 0 <= 1) || defined __sun)
-    {
-      size_t res = iconv (cd, NULL, NULL, &outptr, &outsize);
-
-      if (res == (size_t)(-1))
-	return -1;
-    }
-# endif
-    if (outsize != 0)
-      abort ();
-  }
-
-  return 0;
-#undef tmpbufsize
+  if (context->to_code == po_charset_utf8)
+    /* If a conversion to UTF-8 fails, the problem lies in the input.  */
+    po_xerror (PO_SEVERITY_FATAL_ERROR, context->message, NULL, 0, 0, false,
+	       xasprintf (_("%s: input is not valid in \"%s\" encoding"),
+			  context->from_filename, context->from_code));
+  else
+    po_xerror (PO_SEVERITY_FATAL_ERROR, context->message, NULL, 0, 0, false,
+	       xasprintf (_("\
+%s: error while converting from \"%s\" encoding to \"%s\" encoding"),
+			  context->from_filename, context->from_code,
+			  context->to_code));
+  /* NOTREACHED */
+  abort ();
 }
 
 char *
-convert_string (iconv_t cd, const char *string)
+convert_string (iconv_t cd, const char *string,
+		const struct conversion_context* context)
 {
   size_t len = strlen (string) + 1;
   char *result = NULL;
@@ -184,31 +88,36 @@ convert_string (iconv_t cd, const char *string)
 	&& strlen (result) == resultlen - 1)
       return result;
 
-  error (EXIT_FAILURE, 0, _("conversion failure"));
+  conversion_error (context);
   /* NOTREACHED */
   return NULL;
 }
 
 static void
-convert_string_list (iconv_t cd, string_list_ty *slp)
+convert_string_list (iconv_t cd, string_list_ty *slp,
+		     const struct conversion_context* context)
 {
   size_t i;
 
   if (slp != NULL)
     for (i = 0; i < slp->nitems; i++)
-      slp->item[i] = convert_string (cd, slp->item[i]);
+      slp->item[i] = convert_string (cd, slp->item[i], context);
 }
 
 static void
-convert_msgid (iconv_t cd, message_ty *mp)
+convert_msgid (iconv_t cd, message_ty *mp,
+	       const struct conversion_context* context)
 {
-  mp->msgid = convert_string (cd, mp->msgid);
+  if (mp->msgctxt != NULL)
+    mp->msgctxt = convert_string (cd, mp->msgctxt, context);
+  mp->msgid = convert_string (cd, mp->msgid, context);
   if (mp->msgid_plural != NULL)
-    mp->msgid_plural = convert_string (cd, mp->msgid_plural);
+    mp->msgid_plural = convert_string (cd, mp->msgid_plural, context);
 }
 
 static void
-convert_msgstr (iconv_t cd, message_ty *mp)
+convert_msgstr (iconv_t cd, message_ty *mp,
+		const struct conversion_context* context)
 {
   char *result = NULL;
   size_t resultlen;
@@ -242,27 +151,28 @@ convert_msgstr (iconv_t cd, message_ty *mp)
 	  }
       }
 
-  error (EXIT_FAILURE, 0, _("conversion failure"));
+  conversion_error (context);
 }
 
 #endif
 
 
-void
+bool
 iconv_message_list (message_list_ty *mlp,
 		    const char *canon_from_code, const char *canon_to_code,
 		    const char *from_filename)
 {
   bool canon_from_code_overridden = (canon_from_code != NULL);
+  bool msgids_changed;
   size_t j;
 
   /* If the list is empty, nothing to do.  */
   if (mlp->nitems == 0)
-    return;
+    return false;
 
   /* Search the header entry, and extract and replace the charset name.  */
   for (j = 0; j < mlp->nitems; j++)
-    if (mlp->item[j]->msgid[0] == '\0' && !mlp->item[j]->obsolete)
+    if (is_header (mlp->item[j]) && !mlp->item[j]->obsolete)
       {
 	const char *header = mlp->item[j]->msgstr;
 
@@ -301,10 +211,10 @@ iconv_message_list (message_list_ty *mlp,
 			    && strcmp (charset, "CHARSET") == 0)
 			  canon_charset = po_charset_ascii;
 			else
-			  error (EXIT_FAILURE, 0,
-				 _("\
+			  po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0,
+				     false, xasprintf (_("\
 present charset \"%s\" is not a portable encoding name"),
-				 charset);
+						charset));
 		      }
 		  }
 		else
@@ -312,10 +222,11 @@ present charset \"%s\" is not a portable encoding name"),
 		    if (canon_from_code == NULL)
 		      canon_from_code = canon_charset;
 		    else if (canon_from_code != canon_charset)
-		      error (EXIT_FAILURE, 0,
-			     _("\
+		      po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0,  0,
+				 false,
+				 xasprintf (_("\
 two different charsets \"%s\" and \"%s\" in input file"),
-			     canon_from_code, canon_charset);
+					    canon_from_code, canon_charset));
 		  }
 		freesa (charset);
 
@@ -336,16 +247,19 @@ two different charsets \"%s\" and \"%s\" in input file"),
       if (is_ascii_message_list (mlp))
 	canon_from_code = po_charset_ascii;
       else
-	error (EXIT_FAILURE, 0, _("\
+	po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
+		   _("\
 input file doesn't contain a header entry with a charset specification"));
     }
 
+  msgids_changed = false;
+
   /* If the two encodings are the same, nothing to do.  */
   if (canon_from_code != canon_to_code)
     {
 #if HAVE_ICONV
       iconv_t cd;
-      bool msgids_changed;
+      struct conversion_context context;
 
       /* Avoid glibc-2.1 bug with EUC-KR.  */
 # if (__GLIBC__ - 0 == 2 && __GLIBC_MINOR__ - 0 <= 1) && !defined _LIBICONV_VERSION
@@ -355,39 +269,51 @@ input file doesn't contain a header entry with a charset specification"));
 # endif
       cd = iconv_open (canon_to_code, canon_from_code);
       if (cd == (iconv_t)(-1))
-	error (EXIT_FAILURE, 0, _("\
+	po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
+		   xasprintf (_("\
 Cannot convert from \"%s\" to \"%s\". %s relies on iconv(), \
 and iconv() does not support this conversion."),
-	       canon_from_code, canon_to_code, basename (program_name));
+			      canon_from_code, canon_to_code,
+			      basename (program_name)));
+
+      context.from_code = canon_from_code;
+      context.to_code = canon_to_code;
+      context.from_filename = from_filename;
 
-      msgids_changed = false;
       for (j = 0; j < mlp->nitems; j++)
 	{
 	  message_ty *mp = mlp->item[j];
 
-	  if (!is_ascii_string (mp->msgid))
+	  if ((mp->msgctxt != NULL && !is_ascii_string (mp->msgctxt))
+	      || !is_ascii_string (mp->msgid))
 	    msgids_changed = true;
-	  convert_string_list (cd, mp->comment);
-	  convert_string_list (cd, mp->comment_dot);
-	  convert_msgid (cd, mp);
-	  convert_msgstr (cd, mp);
+	  context.message = mp;
+	  convert_string_list (cd, mp->comment, &context);
+	  convert_string_list (cd, mp->comment_dot, &context);
+	  convert_msgid (cd, mp, &context);
+	  convert_msgstr (cd, mp, &context);
 	}
 
       iconv_close (cd);
 
       if (msgids_changed)
 	if (message_list_msgids_changed (mlp))
-	  error (EXIT_FAILURE, 0, _("\
+	  po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
+		     xasprintf (_("\
 Conversion from \"%s\" to \"%s\" introduces duplicates: \
 some different msgids become equal."),
-		 canon_from_code, canon_to_code);
+				canon_from_code, canon_to_code));
 #else
-	  error (EXIT_FAILURE, 0, _("\
+	  po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
+		     xasprintf (_("\
 Cannot convert from \"%s\" to \"%s\". %s relies on iconv(). \
 This version was built without iconv()."),
-		 canon_from_code, canon_to_code, basename (program_name));
+				canon_from_code, canon_to_code,
+				basename (program_name)));
 #endif
     }
+
+  return msgids_changed;
 }
 
 msgdomain_list_ty *
@@ -401,9 +327,10 @@ iconv_msgdomain_list (msgdomain_list_ty *mdlp,
   /* Canonicalize target encoding.  */
   canon_to_code = po_charset_canonicalize (to_code);
   if (canon_to_code == NULL)
-    error (EXIT_FAILURE, 0,
-	   _("target charset \"%s\" is not a portable encoding name."),
-	   to_code);
+    po_xerror (PO_SEVERITY_FATAL_ERROR, NULL, NULL, 0, 0, false,
+	       xasprintf (_("\
+target charset \"%s\" is not a portable encoding name."),
+			  to_code));
 
   for (k = 0; k < mdlp->nitems; k++)
     iconv_message_list (mdlp->item[k]->messages, mdlp->encoding, canon_to_code,
