@@ -14,7 +14,7 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software Foundation,
-   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -76,7 +76,7 @@ execute_and_read_po_output (const char *progname,
   fclose (fp);
 
   /* Remove zombie process from process list, and retrieve exit status.  */
-  exitstatus = wait_subprocess (child, progname, false, true, true);
+  exitstatus = wait_subprocess (child, progname, false, false, true, true);
   if (exitstatus != 0)
     error (EXIT_FAILURE, 0, _("%s subprocess failed with exit code %d"),
 	   progname, exitstatus);
@@ -108,7 +108,7 @@ msgdomain_read_java (const char *resource_name, const char *locale_name)
      necessary for running the testsuite before "make install".  */
   gettextjar = getenv ("GETTEXTJAR");
   if (gettextjar == NULL || gettextjar[0] == '\0')
-    gettextjar = GETTEXTJAR;
+    gettextjar = relocate (GETTEXTJAR);
 
   /* Assign a default value to the resource name.  */
   if (resource_name == NULL)
