@@ -120,7 +120,7 @@ write_msg (FILE *output_file, message_list_ty *mlp, const char *locale_name)
     {
       message_ty *mp = mlp->item[j];
 
-      if (mp->msgid[0] == '\0')
+      if (is_header (mp))
 	/* Tcl's msgcat unit ignores this, but msgunfmt needs it.  */
 	fprintf (output_file, "set ::msgcat::header ");
       else
@@ -143,6 +143,25 @@ msgdomain_write_tcl (message_list_ty *mlp, const char *canon_encoding,
   if (mlp->nitems == 0)
     return 0;
 
+  /* Determine whether mlp has entries with context.  */
+  {
+    bool has_context;
+    size_t j;
+
+    has_context = false;
+    for (j = 0; j < mlp->nitems; j++)
+      if (mlp->item[j]->msgctxt != NULL)
+	has_context = true;
+    if (has_context)
+      {
+	multiline_error (xstrdup (""),
+			 xstrdup (_("\
+message catalog has context dependent translations\n\
+but the Tcl message catalog format doesn't support contexts\n")));
+	return 1;
+      }
+  }
+
   /* Determine whether mlp has plural entries.  */
   {
     bool has_plural;
